--- nagios-3.0.3/base/notifications.c.orig	2008-06-23 16:47:41.000000000 -0400
+++ nagios-3.0.3/base/notifications.c	2009-07-20 22:47:18.000000000 -0400
@@ -788,6 +788,12 @@
 	int notification_number=0;
 	time_t current_time=0L;
 	service *temp_service=NULL;
+	/* For recovery escalations, used to help determine if any escalation   */
+	/* was sent out with a problem state listed in the escalation to let us */
+	/* know if we need to send out a recovery for cases where the previous  */
+	/* non-ok state is NOT listed in the escalation .                       */
+	int i=0;
+	int send_recovery=FALSE;
 
 	log_debug_info(DEBUGL_FUNCTIONS,0,"is_valid_escalation_for_service_notification()\n");
 
@@ -823,8 +829,72 @@
 		return FALSE;
 
 	/* skip this escalation if the state options don't match */
-	if(svc->current_state==STATE_OK && se->escalate_on_recovery==FALSE)
-		return FALSE;
+	if(svc->current_state==STATE_OK){
+		if (se->escalate_on_recovery==FALSE)
+			return FALSE;
+
+		/* If the escalation specifies one or more alert states (non-ok),  */
+		/* only allow the escalation if the non-ok state from the previous */
+		/* service state is listed in the escalation and if the escalation */
+		/* is NOT the first escalation (as that would mean there was no    */
+		/* non-ok escalation sent before the recovery escalation).         */
+
+		if ((se->escalate_on_unknown!=FALSE) || (se->escalate_on_warning!=FALSE) || (se->escalate_on_critical!=FALSE)){
+			
+			/* If the recovery is the first escalation, we had no previous */
+			/* non-ok escalation go out, so do not send a recovery         */
+			/* notification.                                               */
+        
+			if (svc->current_notification_number==se->first_notification)
+				return FALSE;
+
+			/* Only send out a recovery for the escalation if the previous     */
+			/* (non-ok) service state is a state also listed in the escalation */
+			/* OR we sent out an escalation notification for a problem state   */
+			/* at any time previously within the scope of the escalation       */
+			if (temp_service->last_state==STATE_WARNING && se->escalate_on_warning==TRUE){
+				return TRUE;
+            	}
+			else if (temp_service->last_state==STATE_CRITICAL && se->escalate_on_critical==TRUE){
+				return TRUE;
+            	}
+			else if (temp_service->last_state==STATE_UNKNOWN && se->escalate_on_unknown==TRUE){
+				return TRUE;
+            	}
+			else{
+
+				/* Walk backwards through the service state history (if we   */
+				/* have history for the service) no farther than the value   */
+				/* of first_notification for the escalation.  If any problem */
+				/* state in the service history within the scope of this     */
+				/* escalation caused an escalation notification to be sent,  */
+				/* we then send an escalation recovery notification as well. */
+
+				send_recovery=FALSE;
+
+				if (temp_service->state_history_index > 0){
+					for (i=temp_service->state_history_index; i>=0 && i>=se->first_notification; i--){
+						if (temp_service->state_history[i]==STATE_UNKNOWN && se->escalate_on_unknown==TRUE){
+							send_recovery=TRUE;
+							break;
+							}
+						else if (temp_service->state_history[i]==STATE_WARNING && se->escalate_on_warning==TRUE){
+							send_recovery=TRUE;
+							break;
+							}
+						else if (temp_service->state_history[i]==STATE_CRITICAL && se->escalate_on_critical==TRUE){
+							send_recovery=TRUE;
+							break;
+							}
+
+						}
+
+					}
+					return send_recovery;
+
+				} 
+			}
+		}
 	else if(svc->current_state==STATE_WARNING && se->escalate_on_warning==FALSE)
 		return FALSE;
 	else if(svc->current_state==STATE_UNKNOWN && se->escalate_on_unknown==FALSE)
